/*
 * client.c	SPAMNET Network Tester  Client
 *
 * Copyright 2003 MontaVista Software
 *
 * Author: Steven Dake (sdake@mvista.com)
 *
 * This program is licensed under the MontaVista Software, Inc. License
 * Agreement ("License Agreement"), and is for the purposes of the License
 * Agreement a MontaVista Licensed Deployment Program.  The License requires
 * that you have a valid Product Subscription and Software License Agreement
 * with MontaVista Software, Inc., or are a Named Contact with active access to
 * the MontaVista Zone, or have a Software License Agreement with MontaVista
 * Software, Inc.  This program comes with no warranties other than those
 * provided for in the Product Subscription agreement.  The License Agreement
 * grants you the right to install, modify, and use the program.  You may
 * distribute the object code and scripts for this program, but you have no
 * right to distribute source code for this program.
 */
#include <sys/select.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <linux/if.h>
#include <linux/sockios.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include "dataset.h"

#define DEBUG 1
#define SERVER_BACKLOG 2
#define SERVER_PORT 20000

struct message {
	int dataset;
	int start;
	int size;
};

int send_retry (int s, const void *msg, size_t len, int flags) {
	int index = 0;
	int bytes_sent;
	char *cmsg = (char *)msg;

	do {
		bytes_sent = send (s, &cmsg[index], len, flags);
		if (bytes_sent == -1) {
			return (-1);
		}
		index += bytes_sent;
		len -= bytes_sent;
	} while (len);
}

int main (int argc, char **argv)
{
	int nodeexec_server_fd;
	int client_fd;
	struct sockaddr_in in_addr;

	fd_set read_fds;
	int max_fd;
	struct message message;
	int result;
	int dataset;
	int size;
	int start;


	printf ("MontaVista Software SPAMNET Network Test Client\n");
	printf ("Copyright (C) 2003 MontaVista Software, Inc.\n");

	if (argc != 2) {
		printf ("usage: %s [ip]\n", argv[0]);
	}

	result = inet_pton (AF_INET, argv[1], &in_addr.sin_addr);
	in_addr.sin_family = AF_INET;
	in_addr.sin_port = htons (SERVER_PORT);
	client_fd = socket (AF_INET, SOCK_STREAM, 0);
	result = connect (client_fd, (struct sockaddr *)&in_addr, sizeof (struct sockaddr_in));
	if (result != 0) {
		perror ("Connect: ");
		exit (1);
	}

	for (;;) {
		dataset = random() % DATASETS;
		start = random() % datasets[dataset].size;
		size = random() % (datasets[dataset].size - start);
		message.dataset = htonl (dataset);
		message.start = htonl (start);
		message.size = htonl (size);
		result = send_retry (client_fd, &message, sizeof (struct message), MSG_NOSIGNAL);
		if (result == -1) {
			perror ("send: ");
			exit (1);
		}

		result = send_retry (client_fd, &datasets[dataset].dataset[start], size, MSG_NOSIGNAL);
		if (result == -1) {
			perror ("send: ");
			exit (1);
		}
	}
}

