/*
 * server.c SPAMNET Network Tester Server
 *
 * Copyright 2003 MontaVista Software
 *
 * Author: Steven Dake (sdake@mvista.com)
 *
 * This program is licensed under the MontaVista Software, Inc. License
 * Agreement ("License Agreement"), and is for the purposes of the License
 * Agreement a MontaVista Licensed Deployment Program.  The License requires
 * that you have a valid Product Subscription and Software License Agreement
 * with MontaVista Software, Inc., or are a Named Contact with active access to
 * the MontaVista Zone, or have a Software License Agreement with MontaVista
 * Software, Inc.  This program comes with no warranties other than those
 * provided for in the Product Subscription agreement.  The License Agreement
 * grants you the right to install, modify, and use the program.  You may
 * distribute the object code and scripts for this program, but you have no
 * right to distribute source code for this program.
 */
#include <sys/select.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <linux/if.h>
#include <linux/sockios.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include "dataset.h"

#define DEBUG 1
#define SERVER_BACKLOG 2
#define SERVER_PORT 20000
#define SERVER_HEARTBEATINSEC 3


struct connection {
	int active;
};

int connection_entries = 0;
struct connection *connections = 0;

static inline void grow_connections_table (int fd) {

	if (fd + 1 > connection_entries) {
		connections = realloc (connections, (fd + 1) * sizeof (struct connection));
		if (connections == 0) {
			printf ("out of memory, woops\n");
			exit (1);
		}
		memset (&connections[connection_entries], 0,
		(fd - connection_entries + 1) * sizeof (struct connection));
		connection_entries = fd + 1;
	}
}

char buffer[16384];
struct message {
	int dataset;
	int start;
	int size;
};

unsigned long long total_messages = 0;
unsigned long long total_compared = 0;
unsigned long long total_megs = 0;
unsigned long long one_mb = 1024*1024;

static inline int process_receive_message (int fd)
{
	int result;
	struct message message;
	int size;

	result = recv (fd, &message, sizeof (struct message), MSG_WAITALL | MSG_NOSIGNAL);
	if (result != sizeof (struct message)) {
		return (-1);
	}
	
	message.dataset = ntohl (message.dataset);
	message.start = ntohl (message.start);
	message.size = ntohl (message.size);

	result = recv (fd, buffer, message.size, MSG_WAITALL | MSG_NOSIGNAL);
	if (result != message.size) {
		return (-1);
	}
	if (memcmp (buffer, &datasets[message.dataset].dataset[message.start], message.size) != 0) {
		printf ("Erroneous data compare at message %lld\n", total_messages);
		exit (1);
	}

	/*
	 * Bookeeping
	 */
	total_compared += message.size;
	total_messages += 1;
	if ((total_compared / one_mb) > total_megs) {
		total_megs = total_compared / one_mb;
		printf ("Processed %lld messages, %lld MB\n", total_messages, total_megs);
	}

	return (result);
}

int main (int argc, char **argv)
{
	int nodeexec_server_fd;
	int new_fd;
	struct sockaddr_in in_addr;
	int result;
	fd_set read_fds;
	int max_fd;
	int i;
	int addrlen;

	printf ("MontaVista Software SPAMNET Network Test Server\n");
	printf ("Copyright (C) 2003 MontaVista Software, Inc.\n");

	/*
	 * Create socket for clients, bind to port, listen for connection
	 */
	nodeexec_server_fd = socket (AF_INET, SOCK_STREAM, 0);
	if (nodeexec_server_fd == -1) {
		perror ("Cannot create client connection socket.");
		exit (1);
	}
	in_addr.sin_family = AF_INET;
	in_addr.sin_addr.s_addr = INADDR_ANY;
	in_addr.sin_port = htons (SERVER_PORT);
	result = bind (nodeexec_server_fd, (struct sockaddr *)&in_addr, sizeof (struct sockaddr_in));
	if (result) {
		printf ("Could not bind: %s.\n", strerror (errno));
		exit (1);
	}
	listen (nodeexec_server_fd, SERVER_BACKLOG);

	for (;;) {
		/*
		 * Wait for incoming data
		 */
		FD_ZERO (&read_fds);
		FD_SET (nodeexec_server_fd, &read_fds);

		max_fd = nodeexec_server_fd;
		for (i = 0; i < connection_entries; i++) {
			if (connections[i].active) {
				FD_SET (i, &read_fds);
				if (i > max_fd) {
					max_fd = i;
				}
			}
		}

		result = select (max_fd + 1, &read_fds, NULL, NULL, NULL);
		if (result == -1) {
			perror ("select: ");
			exit (1);
		}

		if (FD_ISSET (nodeexec_server_fd, &read_fds)) {
			addrlen = sizeof (struct sockaddr_in);
			new_fd = accept (nodeexec_server_fd, (struct sockaddr *)&in_addr, &addrlen);
			printf ("connection received from spamnet client %s\n", inet_ntoa (in_addr.sin_addr));
			grow_connections_table (new_fd);
			connections[new_fd].active = 1;
		}

		/*
		 * Check all sockets for data to read, if data, process message
		 */
		for (i = 0; i < connection_entries; i++) {
			if (connections[i].active && FD_ISSET (i, &read_fds)) {
				result = process_receive_message (i);
				/*
				 * Couldn't read any data from socket because it closed
				 */
				if (result == -1) {
					printf ("Closing socket after process_receive_message error\n");
					connections[i].active = 0;
					close (i);
				}
			}
		}
	}
}
